// -----------------------------------------------------------------------------
//
// Copyright 2011(c) Analog Devices, Inc.
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//  - Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  - Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//  - Neither the name of Analog Devices, Inc. nor the names of its
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//  - The use of this software may or may not infringe the patent rights
//    of one or more patent holders.  This license does not release you
//    from the requirement that you obtain separate licenses from these
//    patent holders to use this software.
//  - Use of the software either in source or binary form, must be run
//    on or directly connected to an Analog Devices Inc. component.
//
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// --------------------------------------------------------------------------//
// FILE NAME :      AD7699.v
// MODULE NAME :    AD7699
// AUTHOR : Istvan Csomortani
// AUTHORï¿½S EMAIL : istvan.csomortani@analog.com
// --------------------------------------------------------------------------//
// SVN REVISION: 1863
// KEYWORDS : AD7699, EvalBoard
// --------------------------------------------------------------------------//
// PURPOSE : Module used to retrieve data from the AD7699
// --------------------------------------------------------------------------//
// REUSE ISSUES
// Reset Strategy      : reset on '1'
// Clock Domains       : fpga_clk_i and adc_clk_i are 40MHz
// Critical Timing     :
// Test Features       :
// Asynchronous I/F    :
// Instantiations      :
// Synthesizable (y/n) : Y
// Target Device       : Cyclone II EP2C5F256C7N from AD7699 EvalBoard
// Other               :
// --------------------------------------------------------------------------//

`timescale 100 ps/ 10 ps

//---------------------------------------------------------------------------//
//----------- Module Declaration --------------------------------------------//
//---------------------------------------------------------------------------//
module AD7699 (
//----------- Ports Declarations --------------------------------------------//
// Clock and reset signals
        // System clock
        input               fpga_clk_i,         
        // Clock to be applied to ADC to drive the conversions
        input               adc_clk_i,          
        // Active high reset signal
        input               reset_i,            

// IP control and data interface
        // Active low signal to initiate a data write to the ADC
        input               wr_data_n_i,        
        // Data to be written to the ADC
        input      [15:0]   data_i,             

        // Data read from the ADC    
        output reg [127:0]  data_channels_o,    
        // When set to high the data read from the ADC is available on the data bus
        output reg          data_rd_ready_o,    
        // Used to signal the status of an ADC write: 0 - write in progress, 1 - write complete
        output reg          data_wr_ready_o,    

// External ADC connections
        // Master Input Slave Output
        input               MISO_i,             
        // Master Output Slave Input
        output              MOSI_o,             
        // Serial Data Clock
        output              SCLK_o,             
        // Pin to control the Start of a new Conversion
        output reg          CNV_o               
);

//---------------------------------------------------------------------------//
//----------- Module Parameters ---------------------------------------------//
//---------------------------------------------------------------------------//
// State Machine - State Names (parameters)
// state in which a write can be performed to AD7699
parameter           IDLE             = 10'b0000000001; 
// start the conversion process
parameter           START_CNV        = 10'b0000000010; 
// wait conversion time
parameter           CONVERSION       = 10'b0000000100; 
// if is not RWDAC mode, bring CNV back to high 
parameter           CNV_HIGH         = 10'b0000001000; 
// bring CNV low
parameter           CNV_LOW          = 10'b0000010000; 
// perform read data, before busy is activated
parameter           READ_DATA        = 10'b0000100000; 
// depending on the configuration, selects the next state
parameter           SELECT_CHANNEL   = 10'b0001000000; 
// state in which samples are referenced to COM pin
parameter           SYNCHRONIZE_COM  = 10'b0010000000; 
// state in which samples are differential
parameter           SYNCHRONIZE_DIFF = 10'b0100000000; 
// waits for the conversion time to pass
parameter           WAIT_CYCLE_END   = 10'b1000000000; 
//---------------------------------------------------------------------------//
//Modify this to change the Read/Write Modes
//---------------------------------------------------------------------------//
// RWDC - "0" - Read/Write During Conversion
// RWAC - "1" - Read/Write After Conversion
// RWSC - "2" - Read/Write Spanning Conversion
//---------------------------------------------------------------------------//
parameter           SPI_MODE        = 1;
//---------------------------------------------------------------------------//
//----------------Local Parameters-------------------------------------------//
//---------------------------------------------------------------------------//
// Digital Interface (SPI) Modes
localparam          RWDC            = 0; // R/W During Conversion;
localparam          RWAC            = 1; // R/W After Convertion;
localparam          RWSC            = 2; // R/W Spanning Conversion
// Interface timings
localparam real     FPGA_CLK        = 40;  //MHz
localparam real     ADC_CYCLE_TIME  = 2;   // us
localparam real     ADC_CONV_TIME   = 1.4; // us 
localparam real     ADC_DATA_TIME   = 0.8; // us 
localparam real     ADC_QUITE_TIME  = 0.6; // us 
// Timing apadted to the FPGA's frequency
// Note: If you change the frequency, you have to change the PLL's settings too
localparam integer  CYCLE_CNT       = (FPGA_CLK * ADC_CYCLE_TIME) - 3;    
localparam integer  CONV_CNT        = (FPGA_CLK * ADC_CONV_TIME)  - 2;    
localparam integer  RWDC_CNT        = (FPGA_CLK * ADC_DATA_TIME)  - 2;    
localparam integer  QUITE_CNT       = (SPI_MODE == RWDC) ? ((2 * FPGA_CLK * ADC_QUITE_TIME) - 2) : 
                                                            ((FPGA_CLK * ADC_QUITE_TIME) );

//---------------------------------------------------------------------------//
//----------- Registers Declarations ----------------------------------------//
//---------------------------------------------------------------------------//
// State Machine Registers
// Current State of the State Machine
reg [9:0]           state;                  
// Next State of the State Machine
reg [9:0]           state_ns        = IDLE; 
// Used to generate SCLK signal
reg                 SCLK_en         = 0;    
// Transmitted data, as received from the CED1Z_interface module
reg [15:0]          tx_reg;                 
// Register used for sending data
reg [15:0]          tx_reg_d;               
// Register used for acquiring data
reg [15:0]          rx_reg;                 
// Register used for measuring the cycle time
reg [31:0]          cycle_time_cnt;         
// Register used for measuring the conversion time
reg [31:0]          conv_time_cnt;          
// Register used for measuring the read/write time during conversion
reg [31:0]          rwdc_time_cnt;          
// Register used for measuring the quite time between conversion and acquisition
reg [31:0]          quite_time_cnt;         
// Register used for counting the clock signals sent
reg [5:0]           sclk_time_cnt;          
// Register used for configuring the sequencer
reg [1:0]           seq;                    
reg [1:0]           seq_d;
// Register used for input channel selection
reg [2:0]           inx;                    
reg [2:0]           inx_d;                    
// Register used for input channel configuration
reg [2:0]           incc;                   
reg [2:0]           incc_d;
// Register used for keeping track of the channel acquired
reg [2:0]           channel;                
//Count the conversions
reg                 first_cnv_l     = 0;    

//---------------------------------------------------------------------------//
//----------- Assign/Always Blocks ------------------------------------------//
//---------------------------------------------------------------------------//
// clock is idle LOW
assign SCLK_o           = SCLK_en ? adc_clk_i : 1'b0;   
// send data from tx_reg_d register on the MOSI data line
assign MOSI_o           = tx_reg_d[13];                 

// Process for transmitting/receiving data to/from AD7699 with syncrone reset
always @(negedge adc_clk_i)                             
begin
    if (reset_i == 1'b1)
    begin
        rx_reg          <= 16'h0;
        tx_reg_d        <= 16'h0;
    end
    else if(SCLK_en == 1'b1) 
    begin
        if(SPI_MODE == RWSC)
        begin
            if(state == READ_DATA) 
            begin
                rx_reg      <= { rx_reg[14:0], MISO_i };    
                tx_reg_d    <= { tx_reg_d[14:0], 1'b0 };    
            end
        end
        else
        begin
            rx_reg      <= { rx_reg[14:0], MISO_i };        
            tx_reg_d    <= { tx_reg_d[14:0], 1'b0 };        
        end
    end
    else if (state == IDLE)                     
    begin
        rx_reg      <= 16'h0;
    end
    else if (state == CONVERSION)               
    begin
        tx_reg_d    <= {1'b0, tx_reg[13], tx_reg[13:0]};
    end
end

// Generate counters for timing
always @(negedge fpga_clk_i)            
begin
    if (state == START_CNV ||
        reset_i == 1'b1)     
    begin
        cycle_time_cnt   <= CYCLE_CNT;
        conv_time_cnt    <= CONV_CNT;
        rwdc_time_cnt    <= RWDC_CNT;
        quite_time_cnt   <= QUITE_CNT;
    end
    else
    begin
        // Decrease cycle counter
        if ( cycle_time_cnt != 0 )           
        begin
            cycle_time_cnt <= cycle_time_cnt - 32'h1;
        end
        // Decrease conversion counter
        if ( conv_time_cnt != 0 )            
        begin
            conv_time_cnt    <= conv_time_cnt - 32'h1;
        end
        //Decrease rwdc counter
        if( (rwdc_time_cnt != 0) && (state == CONVERSION) )             
        begin
            rwdc_time_cnt <= rwdc_time_cnt - 32'h1; 
        end
        //Decrease quite time counter
        if( (quite_time_cnt != 0) && (state ==  CNV_HIGH) ) 
        begin
            quite_time_cnt <= quite_time_cnt - 32'h1; 
        end
    end
end

// State Machine - Next State Logic
always @( state, sclk_time_cnt, cycle_time_cnt, incc, conv_time_cnt, rwdc_time_cnt, quite_time_cnt, reset_i, incc_d )
begin

    if(reset_i == 1'b1)
    begin
        state_ns <= IDLE;
    end
    else
    begin
        state_ns <= state;
        case(state)
            IDLE:
            begin
                state_ns    <= START_CNV;
            end
            START_CNV:
            begin
                state_ns    <= CONVERSION;
            end
            CONVERSION:
            begin
                case(SPI_MODE)
                    RWAC:
                    begin
                        if(conv_time_cnt == 32'b0)
                        begin
                            state_ns  <= CNV_LOW;
                        end                    
                    end
                    RWSC, RWDC :
                    begin
                        if(rwdc_time_cnt == 32'b0)
                        begin
                            state_ns <= CNV_HIGH;
                        end
                    end
                endcase
            end
            CNV_HIGH:
            begin
                if(quite_time_cnt == 32'b0)
                begin
                    state_ns <= CNV_LOW;
                end
            end
            CNV_LOW:
            begin
                state_ns    <= READ_DATA;
            end
            READ_DATA:
            begin
                if(SPI_MODE == RWDC)
                begin
                    state_ns  <= SELECT_CHANNEL;    
                end
				else
				if(sclk_time_cnt == 6'b0)
	       	    begin
			        state_ns  <= SELECT_CHANNEL;
					 end
				end
            SELECT_CHANNEL:
            begin
                // Configuration words with differential pairs
                if( incc_d == 3'b000 || 
                    incc_d == 3'b001 || 
                    incc_d == 3'b100 || 
                    incc_d == 3'b101 ) 
                begin
                    state_ns  <= SYNCHRONIZE_DIFF;
                end
                else
                begin
                    state_ns  <= SYNCHRONIZE_COM;
                end
            end
            SYNCHRONIZE_COM:
            begin
                state_ns  <= WAIT_CYCLE_END;
            end
            SYNCHRONIZE_DIFF:
            begin
                state_ns  <= WAIT_CYCLE_END;
            end
            WAIT_CYCLE_END:
            begin
                if (cycle_time_cnt == 32'b0)
                begin
                    state_ns  <= IDLE;
                end
            end
            default:
            begin
                state_ns    <= IDLE;
            end
        endcase
    end
end

//SCK Counter logic with syncronous reset
always @(posedge fpga_clk_i)
begin
  if(reset_i == 1'b1)
  begin
    sclk_time_cnt <= 6'h11;
  end
  else
  begin
    case(state)
        IDLE :
        begin
            sclk_time_cnt <= 6'h11;       
        end
        CONVERSION, READ_DATA : 
        begin
            if( sclk_time_cnt != 0 )
            begin
                sclk_time_cnt    <= sclk_time_cnt - 6'h1;
            end       
        end
        CNV_LOW : 
        begin
             sclk_time_cnt <= 6'h11;
        end
        default : 
            sclk_time_cnt <= sclk_time_cnt;
    endcase
  end
end

// State Machine - Output Logic
always @(negedge fpga_clk_i)
begin
    if(reset_i == 1'b1)
    begin
        tx_reg                      <= 16'h0000;
        data_channels_o [ 127:0]    <= 128'h0;
        first_cnv_l          	    <= 1'b0;
		state 			            <= IDLE;
    end
    else
    begin
        state   <= state_ns;
        case( state )
            IDLE:
            begin
                //Configuration register update
                if (wr_data_n_i == 0)      
                begin
                    tx_reg                      <= data_i;
                    data_wr_ready_o             <= 1'b1;
                    channel                     <= 3'h5;
                    incc                        <= (data_i[13] == 1'b1) ? data_i[12:10] : incc;
                    incc_d                      <= incc;
                    inx                         <= (data_i[13] == 1'b1) ? data_i[9:7] : inx;
                    inx_d                       <= inx;
                    seq                         <= (data_i[13] == 1'b1) ? data_i[2:1] : seq;
                    seq_d                       <= seq;
                    data_channels_o [ 127:0]    <= 128'h0;
                end
                else
                // if no write has been performed, keep DIN low
                begin                   
                    tx_reg      <= 16'h0;
                end
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
                data_rd_ready_o <= 1'b0;
            end
            START_CNV:
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b1;
                data_rd_ready_o <= 1'b0;
            end
            // The CNV pin is kept high until EOC. Configuration without busy indicator
            CONVERSION:                 
            begin
                data_wr_ready_o <= 1'b0;
                data_rd_ready_o <= 1'b0;
                if(SPI_MODE == RWAC)
                begin
                    SCLK_en         <= 1'b0;
                    CNV_o           <= 1'b1;
                end
				else
                begin
                    CNV_o           <= 1'b0;
                    if(sclk_time_cnt != 0)
                    begin
                        if(SPI_MODE == RWSC &&
                            first_cnv_l == 1'b0)
                        begin
                            SCLK_en     <= 1'b0;
                        end
                        else
                        begin      
                            SCLK_en     <= 1'b1;
                        end
                    end
                    else
                    begin
                        SCLK_en         <= 1'b0;
                    end      
                end
            end
            CNV_HIGH:
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b1;
                data_rd_ready_o <= 1'b0;
                if(first_cnv_l == 1'b0)
                begin
                    first_cnv_l <= first_cnv_l  + 1'b1;    
                end
             end
            CNV_LOW:
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
                data_rd_ready_o <= 1'b0;
            end
            READ_DATA:
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b1;
                CNV_o           <= 1'b0;
                data_rd_ready_o <= 1'b0;
                if( (SPI_MODE == RWAC) || (SPI_MODE == RWSC) )
                begin
                    SCLK_en         <= 1'b1;
                end
                else
                begin
                   SCLK_en         <= 1'b0; 
                end
            end
            SELECT_CHANNEL:
            begin
                channel         <= channel + 3'h1;
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                data_rd_ready_o <= 1'b0;
                CNV_o           <= 1'b0;
            end
            // Using 11 for channel sequencer on and 00 for channel sequencer disabled. 
            //01 and 10 should not be used to configure the AD7699
            SYNCHRONIZE_COM : 
            begin
                case( channel )
                    3'h0:
                    begin
                        data_channels_o[127:112] <= rx_reg;
                        if ( seq_d == 2'b11  && inx_d == 3'h0)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h1:
                    begin
                        data_channels_o[111:96] <= rx_reg;
                        if ( seq_d == 2'b11 && inx_d == 3'h1)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h2:
                    begin
                        data_channels_o[95:80]  <= rx_reg;
                        if (seq_d == 2'b11 && inx_d == 3'h2)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h3:
                    begin
                        data_channels_o[79:64]  <= rx_reg;
                        if (seq_d == 2'b11 && inx_d == 3'h3)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h4:
                    begin
                        data_channels_o[63:48]  <= rx_reg;
                        if (seq_d == 2'b11 && inx_d == 3'h4)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h5:
                    begin
                        data_channels_o[47:32]  <= rx_reg;
                        if (seq_d == 2'b11 && inx_d == 3'h5)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h6:
                    begin
                        data_channels_o[31:16]  <= rx_reg;
                        if (seq_d == 2'b11 && inx_d == 3'h6)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    default:
                    begin
                        data_channels_o[15:0]   <= rx_reg;
                        data_rd_ready_o         <= 1'b1;
                        channel                 <= 3'h7;
                    end
                endcase
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
            end
            SYNCHRONIZE_DIFF :
            begin
                case( channel )
                    3'h0:
                    begin
                        data_channels_o[127:112] <= rx_reg;
                        if (seq_d == 2'b11 && (inx_d == 3'h0 || inx_d == 3'h1))
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h1:
                    begin
                        data_channels_o[111:96]  <= rx_reg;
                        if ( seq_d == 2'b11 && (inx_d == 3'h2 || inx_d == 3'h3))
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h2:
                    begin
                        data_channels_o[95:80]  <= rx_reg;
                        if (seq_d == 2'b11 && (inx_d == 3'h4 || inx_d == 3'h5))
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h6:
                    begin
                        channel             <= 3'h6; 
                    end
                    default:
                    begin
                        data_channels_o[79:64]  <= rx_reg;
                        data_rd_ready_o         <= 1'b1;
                        channel                 <= 3'h7;
                    end
                endcase
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
            end
            WAIT_CYCLE_END :
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
                data_rd_ready_o <= 1'b0;
            end
        endcase
    end
end

endmodule
