/*
* Simulation model for the ad7689 IC
* General timing without a busy indicator
* */

`timescale 100 ps / 10 ps
module ad7689_model(
    input           din_i,
    input           sck_i,
    input           cnv_i,

    output          sdo_o,
    //FOR DEBUG
    output      [5:0]   state_o,
    output      [7:0]   sck_counter,
    output  reg [15:0]  data_in   
);

// State Machine - State Names (parameters)
parameter POWER_UP      = 6'b000001;    // Initial Power Up State
parameter CNV_HIGH      = 6'b000010;    // 10ns cnv_i high before start cnv
parameter CONVERSION    = 6'b000100;    
parameter QUIET         = 6'b001000;
parameter SDO_VALID_1   = 6'b010000;
parameter ACQUISITION   = 6'b100000; 
//Timing parameters - Should be corespond with the values from the datasheet
parameter T_CONV_MAX    = 16000;
parameter T_CONV_MIN    = 8100;
parameter T_DATA        = 12000;
parameter T_CYC         = 2000;
parameter T_PULSE       = 100;
parameter T_CLSK        = 100;
parameter T_QUIET       = 4000;
parameter FPGA_CLK_PRD  = 250; // 25 ns

localparam MODE          = 1;

reg [5:0] state = POWER_UP;  // state
reg busy = 0;

integer sck_rising_cnt = 0; //SCK rising edge counter
integer sck_falling_cnt = 0; //SCK falling counter

//The Read/Write MODE could be RWDC = 0; RWAC = 1; RWSC = 2;
localparam RWDC     = 0;   //Read/Write during conversion
localparam RWAC     = 1;   //Read/Write after conversion
localparam RWSC     = 2;   //Read/Write spanning conversion

/* To check the minimum pulse width */
time CNVRisingTime  = 0; 
time CNVFallingTime = 0;
time CNVFallingTimeAcq = 0;
time SCKLastFallingEdg = 0;
time SOCTime = 0;
/* To check the sck period */
time SCKRisingTime  = 0;
time SCKFallingTime = 0;

reg [15:0] data;        // data register of the ADC
reg [15:0] new_data;    // register used for new data acquirements
reg [15:0] cfg;         // configuration register of the ADC

reg cnv_buff = 0;
reg sdo_buff;

//FOR DEBUG
assign state_o = state; //Output logic for state 
assign sck_counter = sck_falling_cnt;

initial
begin
    state               = POWER_UP;
    new_data            = $random();
    data                = 16'h8000;
    cfg                 = 16'h0000;
    sdo_buff            = 16'b0;
    #70 sck_falling_cnt = 0;
end

always @(cnv_i) // asyncronos state machine, switching by the edges of the CNV
begin
    if(!cnv_buff && cnv_i)      //Rising edge of the cnv_i
    begin
        cnv_buff = cnv_i;
        CNVRisingTime = $time;
        case(state)
            POWER_UP : 
            begin
                if(MODE == RWDC || MODE == RWSC)
                begin
                  state = CNV_HIGH;
                end
                else
                begin
                  state = CONVERSION;
                  //#T_DATA;
                  //state = QUIET;
                end
                SOCTime = $time;
            end
            CONVERSION :
            begin
               if(MODE == RWDC || MODE == RWSC)
                begin
                    if(CNVRisingTime - SOCTime > T_DATA) 
                    begin
                        $display("AD7699 ERROR %t : Data WR/RD operations during conversion is bigger then %t < %t", $time, T_DATA, $time - SOCTime);
                    end
                    if(sck_falling_cnt != 0)
                    begin
                        $display("AD7699 ERROR %t : Too few SCK falling edge detected on CONVERTION state.", $time);
                    end
                    state = QUIET;
                    //#T_QUIET;
                    //state = ACQUISITION;
                end
                else
                begin
                    $display("AD7699 ERROR %t : CNV rising edge at invalid state.", $time);
                end
            end
            QUIET : 
            begin
                $display("AD7699 ERROR : Invalid CNV rising edge at QUIET state.");
            end
            ACQUISITION : 
            begin
                if($time - SOCTime < T_CYC) 
                begin    
                    $display("AD7699 ERROR %t : Time between conversion is less than %dns", $time, T_CYC);
                end
                if(sck_falling_cnt != 0)
                begin
                    $display("AD7699 ERROR %t : Too few SCK falling edge detected on ACQUISITION state.", $time);
                    sck_falling_cnt = 0;
                end
                SOCTime = $time;
                if(MODE == RWDC || MODE == RWSC)
                begin
                  state = CNV_HIGH;
                end
                else
                begin
                  state = CONVERSION;
                  #T_DATA;
                  state = QUIET;
                end
            end
            default :
            begin
                $display("AD7699 ERROR %t : INVALID CNV Rising edge detected!", $time);
            end         
        endcase    
    end
    else if(cnv_buff && !cnv_i) //Falling edge of the cnv_i
    begin
        cnv_buff = cnv_i;
        CNVFallingTime = $time; 
        case(state)
            CNV_HIGH : 
            begin
                if(CNVFallingTime - CNVRisingTime < T_PULSE) 
                begin
                    $display("AD7699 ERROR %t : CNV Pulse width is too small - %t!", $time, (CNVFallingTime - CNVRisingTime));
                end
                state = CONVERSION;
            end
            ACQUISITION : 
            begin
                if( (CNVFallingTime - SOCTime) >= T_CONV_MAX)
                begin
                    $display("AD7699 ERROR %t : Conversion time is bigger than the maximum conversion time. - %t < %t!", $time, T_CONV_MAX, (CNVFallingTime - SOCTime) ); 
                end
                CNVFallingTimeAcq = $time;
            end
            QUIET :
            begin
                if((CNVFallingTime - SOCTime) <= T_CONV_MIN)
                begin
                    $display("AD7699 ERROR %t : Conversion time is less than the minimum conversion time. - %t > %t!", $time, T_CONV_MIN, (CNVFallingTime - SOCTime) ); 
                end
                state = ACQUISITION;    
            end
            CONVERSION : 
            begin
                if( (CNVFallingTime - SOCTime) < T_CONV_MIN || 
                    (CNVFallingTime - SOCTime) > T_CONV_MAX )
                begin
                    $display("AD7699 ERROR %t : Conversion time is out of range. - (%t , %t) != %t!", $time, T_CONV_MIN, T_CONV_MAX, (CNVFallingTime - SOCTime) ); 
                end
                state = ACQUISITION;    
            end
            default : 
            begin
                $display("AD7699 ERROR %t : INVALID CNV Falling edge detected!", $time);
            end
        endcase
    end
end

//always module to generate a new random number before the next convertion
always @(state)
begin
    if(state == ACQUISITION)
    begin
        new_data = $random();
        data_in = new_data;
    end
end

//always block to generate the input data, shifting the value of the new_data to sdo_o
always @(posedge sck_i)
begin
    sdo_buff <= new_data[15];                       // MSB first at posedge sck
end

assign sdo_o = (cnv_i == 1'b1) ? 1'bZ : sdo_buff;

always @(posedge sck_i)
begin
    if(MODE == RWAC && cnv_i != 1'b0) 
    begin
        $display("AD7699 ERROR %t : SCK reception in INVALID ADC state", $time);
    end
    case(state)
        POWER_UP :
        begin
            $display ("AD7699 ERROR %t : SCLK has been activate in POWER_UP state. This should not happen", $time);
        end
        CNV_HIGH :
        begin
            $display ("AD7699 ERROR %t : SCLK has been activate in CNV_HIGH state. This should not happen", $time);
        end
        CONVERSION:
        begin
            new_data <= {new_data [14:0], new_data[15]};
        end
        QUIET:
        begin
            $display ("AD7699 ERROR %t : SCLK has been activate in QUIET state.This should not happen", $time);
        end
        ACQUISITION:
        begin
            new_data  <= { new_data [15:0], new_data[15] };
            if(sck_rising_cnt == 0) // Check the timing in the first SCK edge
            begin
                if($time - CNVFallingTimeAcq < T_CLSK) 
                begin
                    $display("AD7699 ERROR %t : CNV Low to SCK rising edge is smaller than %dns", $time, T_CLSK);
                end
                sck_rising_cnt <= sck_rising_cnt + 1;
            end
            else if(sck_rising_cnt == 15)
            begin
                sck_rising_cnt <= 0;
            end
            else
            begin
                sck_rising_cnt <= sck_rising_cnt + 1;
            end
        end
    endcase
end

always @(negedge sck_i)
begin
    if(sck_falling_cnt == 15)
    begin
        sck_falling_cnt <= 0;
        SCKLastFallingEdg <= $time;
    end
    else
    begin
       sck_falling_cnt <= sck_falling_cnt + 1; 
    end
end

//always block to check the sclk frequency
always @(posedge sck_i)
begin
  if( (sck_falling_cnt > 1) && (sck_falling_cnt < 14) )
  begin
    if($time - SCKRisingTime != FPGA_CLK_PRD)
    begin 
      $display("AD7699 ERROR %t : SCK frequency missmatch! %t != %t", $time, FPGA_CLK_PRD, ($time - SCKRisingTime));
    end
  end
  SCKRisingTime = $time;
end

endmodule
